unit Mru;

interface

{ $Define Internal}

Uses Classes, Menus, WinProcs, WinTypes, Dialogs, Forms,
     {$IfDef Internal}TLIni{$Else}IniFiles{$EndIf};

{$IfnDef Internal}
Type TExtIniFile = TIniFile;
const MyIniFile = Nil;
{$EndIf}

Type TMruPosition = (mpAfter, mpBefore, mpNewMenu);

Type TMruStyle = (msNormal, msHideSelected, msCheckSelected);

Type TMruEvent = Procedure (Sender: TObject; const Item: String) of Object;

Type TMruOptionsType = (mrLineAfter,             { Linie hinter Liste falls <> leer }
                        mrLineBefore,            { Linie vor Liste falls <> leer }
                        mrShortFileNames,        { nur Datei ohne Pfad }
                        mrCheckFileExist,        { nur anzeigen falls Datei existiert }
                        mrShowNumber,            { &n in Men }
                        mrTopSelected,           { ausgewhle nach oben }
                        mrAutoSave,              { automatisches speichern in ini }
                        mrAutoLoad,              { automatisches laden aus ini }
                        mrDisableAnchorIfEmpty,  { Anchor disablen falls = leer }
                        mrHideAnchorIfEmpty);    { Anchor verstecken falls = leer }

Type TMruOptions = Set of TMruOptionsType;

Type TMruList = Class (TComponent)
                   Private
                      fList: TStrings;
                      fMenuItems: TList;
                      fIniFile: String;
                      fIniSection: String;
                      fAnchor: TMenuItem;
                      fMagic: Longint;
                      fOnMruClick: TMruEvent;
                      fOptions: TMruOptions;
                      fStyle: TMruStyle;
                      fPosition: TMruPosition;
                      fMaxSaveCount: Integer;
                      fMaxShowCount: Integer;
                      Procedure SetIniFile (const S: String);
                      Procedure SetIniSection (const S: String);
                      Function EntryIndex (const S: String): Integer;
                      Function OpenIniFile (var Ini: TExtIniFile): Boolean;
                      Procedure CloseIniFile (Ini: TExtIniFile);
                      Procedure Click (Sender: TObject);
                   Protected
                      TopLineItem: TMenuItem;
                      BottomLineItem: TMenuItem;
                      Procedure UpdateMenus;
                      Procedure ReadIniFile;
                      Procedure WriteIniFile;
                      procedure ClearMenuItems;
                      Function ValidAnchor: Boolean;
                      procedure SetMenuItems;
                      Procedure Loaded; override;
                      Procedure CheckState;
                      Procedure SetOptions (Opt: TMruOptions);
                      Procedure SetStyle (Style: TMruStyle);
                      Procedure SetPosition (Pos: TMruPosition);
                      Procedure SetAnchor (AnAnchor: TMenuItem);
                      Procedure SetMaxShowCount (Cnt: Integer);
                      Procedure SetVisible (Obj: TObject; IsVisible: Boolean);
                   Public
                      constructor Create(AOwner: TComponent); override;
                      destructor Destroy; override;
                      Procedure OpenEntry (const S: String);
                      Procedure CloseEntry (const S: String);
                   Published
                      property MaxSaveCount: Integer Read fMaxSaveCount write fMaxSaveCount Default 50;
                      property MaxShowCount: Integer Read fMaxShowCount write SetMaxShowCount Default 9;
                      property IniFile: String Read fIniFile write SetIniFile;
                      Property IniSection: String Read fIniSection write SetIniSection;
                      property Anchor: TMenuItem Read fAnchor write SetAnchor;
                      Property OnMruClick: TMruEvent Read fOnMruClick write fOnMruClick;
                      Property Options: TMruOptions Read fOptions write SetOptions
                               default [mrLineBefore,mrCheckFileExist,mrShowNumber,mrTopSelected,mrAutoSave,mrAutoLoad];
                      Property Style: TMruStyle Read fStyle write SetStyle default msHideSelected;
                      Property Position: TMruPosition Read fPosition Write SetPosition default mpAfter;
                end;

procedure Register;

implementation

Uses SysUtils;

{$IfDef Win32}
   {$R MRU32.Res}
{$Else}
   {$R *.Res}
{$EndIf}

constructor TMruList.Create(AOwner: TComponent);
begin
   Inherited Create(AOwner);
   fList:=TStringList.Create;
   fMenuItems:=TList.Create;
   fMagic:=GetTickCount;
   Repeat Until GetTickCount<>fMagic;
   fStyle:=msHideSelected;
   fPosition:=mpAfter;
   fOptions:=[mrLineBefore,mrCheckFileExist,mrShowNumber,mrTopSelected,mrAutoSave,mrAutoLoad];
   fMaxSaveCount:=50;
   fMaxShowCount:=9;
end;


Procedure TMruList.SetIniFile (const S: String);
begin
   fIniFile:=S;
   if not (csLoading in ComponentState) and (mrAutoLoad in Options) then
   begin
      ReadIniFile;
      UpdateMenus
   end;
end;

Procedure TMruList.SetIniSection (const S: String);
begin
   fIniSection:=S;
   if not (csLoading in ComponentState) and (mrAutoLoad in Options) then
   begin
      ReadIniFile;
      UpdateMenus
   end;
end;

Procedure TMruList.WriteIniFile;
var I: Integer;
    Ini: TExtIniFile;
begin
   if OpenIniFile(Ini) then With Ini do
   begin
      EraseSection(IniSection);
      WriteInteger(IniSection,'Count',fList.Count);
      For I:=0 to fList.Count-1 do
         WriteString(IniSection,IntToStr(I+1),fList[I]);
   end;
   CloseIniFile(Ini);
end;

Procedure TMruList.ReadIniFile;
var I: Integer;
    S: String;
    Ini: TExtIniFile;
begin
   if OpenIniFile(Ini) then With Ini do
   begin
      fList.Clear;
      I:=ReadInteger(IniSection,'Count',0);
      For I:=1 to i do
      begin
         S:=ReadString(IniSection,IntToStr(I),'');
         if (S<>'') and (not (mrCheckFileExist in Options) or FileExists(S)) then fList.Add(S);
      end;
      UpdateMenus;
      CloseIniFile(Ini);
   end;
end;


Destructor TMruList.Destroy;
begin
   if mrAutoSave in Options then WriteIniFile;
   fList.Free;
   fMenuItems.Free;
   Inherited Destroy;
end;

Function TMruList.EntryIndex (const S: String): Integer;
var I: Integer;
begin
   for I:=0 to fList.Count-1 do
   begin
      if AnsiCompareText(S,fList[i])=0 then
      begin
         Result:=I;
         Exit;
      end;
   end;
   Result:=-1;
end;

Procedure TMruList.OpenEntry (const S: String);
var I: Integer;
begin
   I:=EntryIndex(S);
   if I<0 then
   begin
      if fList.Count>MaxSaveCount then fList.Delete(fList.Count-1);
      I:=fList.Count;
      fList.Add(S);
   end;
   if mrTopSelected in Options then
   begin
      fList.Move(I,0);
      I:=0;
   end;
   fList.Objects[I]:=Pointer(1);
   UpdateMenus;
end;

Procedure TMruList.CloseEntry (const S: String);
var I: Integer;
begin
   if S='' then Exit;
   I:=EntryIndex(S);
   if I>=0 then fList.Objects[I]:=Pointer(0);
   UpdateMenus;
end;

procedure TMruList.ClearMenuItems;
var I: Integer;
begin
   if ValidAnchor Then with Anchor.Parent as TMenuItem do
   begin
      For I:=0 to fMenuItems.Count-1 do
      begin
         SetVisible(fMenuItems[i],False);
         TMenuItem(fMenuItems[i]).Checked:=False;
      end;
      SetVisible(TopLineItem,False);
      SetVisible(BottomLineItem,False);
   end;
end;

procedure TMruList.SetMenuItems;

    Function AddItems: Integer;
    var I, VCount: Integer;
        TM: TMenuItem;
    begin
       Result:=0;
       i:=0;
       VCount:=0;
       While (VCount<MaxShowCount) and (I<fList.Count) do
       begin
          if (Style=msHideSelected) and (Longint(fList.Objects[i])=1) or
             ((mrCheckFileExist in Options) and not FileExists(fList[i])) then
          begin
             Inc(i);
             Continue;
          end;
          TM:=fMenuItems[VCount];
          if mrShortFileNames in Options then TM.Caption:=ExtractFileName(fList[i])
                                         else TM.Caption:=fList[i];
          if (Style=msCheckSelected) and (Longint(fList.Objects[i])=1) then TM.Checked:=True;
          if (mrShowNumber in Options) then TM.Caption:='&'+IntToStr(VCount+1)+'  '+TM.Caption;
          TM.Tag:=I;
          TM.Visible:=True;
          Inc(VCount);
          Inc(i);
          Inc(Result);
       end;
    end;

begin
   if ValidAnchor then
   begin
      if AddItems>0 then
      begin
         SetVisible(TopLineItem,True);
         SetVisible(BottomLineItem,True);
         if (mrDisableAnchorIfEmpty in Options) then Anchor.Enabled:=True;
         if (mrHideAnchorIfEmpty in Options) then Anchor.Visible:=True;
      end
      else begin
         if (mrDisableAnchorIfEmpty in Options) then Anchor.Enabled:=False;
         if (mrHideAnchorIfEmpty in Options) then Anchor.Visible:=False;
      end;
   end;
end;

Function TMruList.ValidAnchor: Boolean;
begin
   Result:=(Anchor<>Nil) and ((Anchor.Parent<>Nil) or (Position=mpNewMenu)) and (Anchor.Owner<>Nil)
            and not (csDesigning in ComponentState);
end;

Procedure TMruList.UpdateMenus;
begin
   ClearMenuItems;
   SetMenuItems;
end;

Function TMruList.OpenIniFile (var Ini: TExtIniFile): Boolean;
begin
   if (IniSection='') or (csDesigning in ComponentState) then Result:=False
   else begin
      Result:=True;
      {$IfDef Internal}
      if IniFile<>'' then Ini:=TExtIniFile.Create(IniFile,'')
                     else Ini:=MyIniFile;
      {$Else}
      if IniFile<>'' then Ini:=TExtIniFile.Create(IniFile)
                     else Ini:=TExtIniFile.Create(ChangeFileExt(ParamStr(0),'.INI'));
      {$EndIf}
   end;
end;

Procedure TMruList.CloseIniFile (Ini: TExtIniFile);
begin
   if (IniSection<>'') and not (csDesigning in ComponentState) and (Ini<>MyIniFile) then Ini.Free;
end;

Procedure TMruList.Click (Sender: TObject);
begin
   if (Sender is TMenuItem) and Assigned(fOnMruClick) then
   begin
      fOnMruClick(Self,fList[(Sender as TMenuItem).Tag]);
   end;
end;

Procedure TMruList.CheckState;
begin
   if (ComponentState*[csDesigning,csLoading]=[]) then Raise Exception.Create('Operation ist nur im Design-Modus mglich!');
end;

Procedure TMruList.SetOptions (Opt: TMruOptions);
begin
   CheckState;
   fOptions:=Opt;
end;

Procedure TMruList.SetStyle (Style: TMruStyle);
begin
   CheckState;
   fStyle:=Style;
end;

Procedure TMruList.SetPosition (Pos: TMruPosition);
begin
   CheckState;
   fPosition:=Pos;
end;

Procedure TMruList.SetAnchor (AnAnchor: TMenuItem);
begin
   CheckState;
   fAnchor:=AnAnchor;
end;

Procedure TMruList.SetMaxShowCount (Cnt: Integer);
begin
   CheckState;
   fMaxShowCount:=Cnt;
end;

Procedure TMruList.Loaded;
var Index: Integer;
    ParentItem: TMenuItem;
    OrgIndex: Integer;

    Function GetLineItem (C: Char): TMenuItem;
    begin
       Result:=TMenuItem.Create(Anchor.Owner);
       Result.Name:='_'+IntToStr(fMagic)+C;
       Result.Caption:='-';
    end;

    Procedure AddItems;
    var VCount: Integer;
        TM: TMenuItem;
    begin
       VCount:=0;
       While (VCount<MaxShowCount) do
       begin
          TM:=TMenuItem.Create(Anchor.Owner);
          TM.Name:='_'+IntToStr(fMagic)+IntToStr(VCount);
          TM.OnClick:=Self.Click;
          ParentItem.Insert(Index,TM);
          fMenuItems.Add(TM);
          Inc(Index);
          Inc(VCount);
       end;
    end;

begin
   Inherited Loaded;
   if csDesigning in ComponentState then Exit;
   if ValidAnchor then
   begin
      if Position=mpNewMenu then
      begin
         ParentItem:=Anchor;
         Index:=0;
      end
      else begin
         ParentItem:=TMenuItem(Anchor.Parent);
         Index:=ParentItem.IndexOf(Anchor);
         if Position=mpAfter then Inc(Index);
      end;
      OrgIndex:=Index;
      AddItems;
      if mrLineAfter in Options then
      begin
         BottomLineItem:=GetLineItem('P');
         ParentItem.Insert(Index,BottomLineItem);
      end;
      if mrLineBefore in Options then
      begin
         TopLineItem:=GetLineItem('A');
         ParentItem.Insert(OrgIndex,TopLineItem);
      end;
   end
   else Raise Exception.Create ('Invalid Anchor!');
   if mrAutoLoad in Options then ReadIniFile;
   UpdateMenus;
end;

Procedure TMruList.SetVisible (Obj: TObject; IsVisible: Boolean);
begin
   if (Obj<>Nil) and (Obj is TMenuItem) then (Obj as TMenuItem).Visible:=IsVisible;
end;


procedure Register;
begin
  RegisterComponents('GV', [TMruList]);
end;

end.



